// Slack API Interface

#ifndef CHAT_SLACK_INCLUDED
#define CHAT_SLACK_INCLUDED
#include <curl/curl.h>

#include "ed_array.h"
#include "string.h"

typedef struct {
    CURL *curl;
} slack_client;

void _slack_debug_print_auth_test(slack_client *client, string token, string cookie);

slack_client slack_init_client();

#ifdef CHAT_SLACK_IMPLEMENTATION

static size_t write_memory_callback(void *contents, size_t size, size_t nmemb,
                                    void *userp) {
    size_t real_size = size * nmemb;
    pushArrayMulti(uint8_t, userp, contents, real_size);
    return real_size;
}

void _slack_debug_print_auth_test(slack_client *client, string token,
                                  string cookie) {
    if (client->curl) {
        struct curl_slist *headers = NULL;
        headers = curl_slist_append(headers, "Content-Type: application/json");
        // headers = curl_slist_append(headers, cookie);

        curl_easy_setopt(client->curl, CURLOPT_URL,
                         "https://slack.com/api/auth.test");

        curl_easy_setopt(client->curl, CURLOPT_HTTPAUTH, (long)CURLAUTH_BEARER);
        // curl_easy_setopt(client->curl, CURLOPT_XOAUTH2_BEARER, token);
        curl_easy_setopt(client->curl, CURLOPT_USERAGENT,
                         "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; "
                         "rv:76.0) Gecko/20100101 Firefox/76.0");
        curl_easy_setopt(client->curl, CURLOPT_HTTPHEADER, headers);

        curl_easy_setopt(client->curl, CURLOPT_POST, 1L);
        curl_easy_setopt(client->curl, CURLOPT_POSTFIELDS, "{}");

        curl_easy_setopt(client->curl, CURLOPT_FOLLOWLOCATION, 1L);

        array(uint8_t) chunk = newArray(uint8_t, 10000);

        curl_easy_setopt(client->curl, CURLOPT_WRITEFUNCTION,
                         write_memory_callback);
        curl_easy_setopt(client->curl, CURLOPT_WRITEDATA, (void *)&chunk);

        /* Perform the request, res gets the return code */
        CURLcode res = curl_easy_perform(client->curl);
        /* Check for errors */
        if (res != CURLE_OK) {
            fprintf(stderr, "curl_easy_perform() failed: %s\n",
                    curl_easy_strerror(res));

        } else {
            fprintf(stderr, "Got data\n%.*s\n", (int)chunk.size, chunk.data);
        }

        free(chunk.data);
    }
}

slack_client slack_init_client(string token, string cookie) {
    CURL *curl = curl_easy_init();

    slack_client client = (slack_client){.curl = curl};

    return client;
}

#endif
#endif
